<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kbruch" version="1">
<MenuBar>
  <Menu noMerge="0" name="file">
    <text>&amp;File</text>
    <Action name="NewTask"/>
    <Action name="Back"/>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
</MenuBar>
<ToolBar name="exerciseToolBar" noEdit="false" fullWidth="true">
  <Action name="NewTask"/>
  <Action name="Back"/>  
  <Separator/>
  <Action name="Hint"/>  
</ToolBar>
</gui>
