<?xml version="1.0"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="kbruch" version="1">
<MenuBar>
  <Menu noMerge="0" name="file">
    <text>&amp;File</text>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
</MenuBar>
</gui>
